function SBMPipeMorris_Dynamic
% Calculate the dynamic particle distribution for a steady flow through 
% a tube using the suspension balance model.  This version of the code
% uses a small offset of the shear rate to alleviate the singularity
% at the center of the tube.
%------------------------------------------------------------
% Solve 
% \partial \phi/ \partial t=  (2 a^2/9 R^2) (1/r) \partial r flux / \partial r
%------------------------------------------------------------ 
% flux= (1-\phi)^5 * \lambda_2* \partial (\eta_n \gammadot) / \partial r +
% (\lambda_2-\lambda_3) \eta_n \gammadot/r)
%------------------------------------------------------------  
% length-scale = R = pipe radius
% velocity-scale = Umean = Q/S = flow-rate / pipe section
%------------------------------------------------------------
% Correlations from rheology proposed in 
% Morris, J., & Boulay, F. (1999) Journal of Rheology, 43(5), 1213?1237.
%------------------------------------------------------------
%% Clear
%------------------------------------------------------------
close all;
%------------------------------------------------------------
%% Initial data
%------------------------------------------------------------
% Ratio a/R  (particle/pipe)
ratioaR=.201/1.64; % 2mm particles
%ratioaR=.105/1.64; % 1mm particles
% Coefficient
eps = (2/9)*ratioaR^2;
% Lambda2
lambda2= 0.8;
% Lambda3
lambda3= 0.5;
% Discretization
N=100; 
% Initial volume fraction
phi0=0.3;
Fi0 = phi0*ones(N+1,1);
% Total time step
tmax=50;
% Time step
dt=5;
% Time-range of integration
tspan = 0:dt:tmax; 
% Space-range of integration
r=(0:1/N:1)';
% parameters 
param=[eps;N;ratioaR;lambda2;lambda3];
%------------------------------------------------------------
%% Solve the problem using ode
%------------------------------------------------------------ 
% T: Column vector of time points 
% X: Solution array
% Each row in X corresponds 
% to the solution at a time 
% returned in the corresponding row of T
% The solver passes parameters
% options = odeset('RelTol',1e-5);
% ode15s stiff problem
options = odeset('RelTol',1e-5);
[~,Fi]=ode15s(@DFlux,tspan,Fi0,options,param);
%------------------------------------------------------------
%% Plot volume fraction
%------------------------------------------------------------
% Evolution of \phi in the pipe
figure;
for k = 1:1:length(tspan)
   % Plot from 0 to 1
   plot(r,Fi(k,:),'-r','LineWidth',2);
   hold on;
   % Plot symmetric part from 0 to -1
   plot(-r,Fi(k,:),'-r','LineWidth',2);
   hold on;
end
title('Evolution of \phi in the pipe', 'FontSize',14);
xlabel('r/R', 'FontSize',16); 
ylabel('\phi', 'FontSize',16);
grid on,
axis([-1 1 0 0.6]);
% % Evolution of \phi at half from middle
% figure;
% plot(tspan,Fi(:,round(N/2)),'-r','LineWidth',2);
% grid on, 
% axis([0 tmax min(Fi(:,round(N/2))) max(Fi(:,round(N/2)))]);
% title('Evolution of \phi at half from middle', 'FontSize',16);
% xlabel('normalized t', 'FontSize',16);
% ylabel('\Phi', 'FontSize',16);
%------------------------------------------------------------
%% Plot velocity
%------------------------------------------------------------
% Evolution of U in the pipe
figure;
UU = zeros(length(tspan),length(r));
for k=1:1:length(tspan)
   [U,~] = PipeAlgebric(Fi(k,:)',r,ratioaR);
   % Plot from 0 to 1
   plot(r,U, '-b','LineWidth',2);
   UU(k,:)=U;
   hold on;
   % Plot symmetric part from 0 to -1
   plot(-r,U, '-b','LineWidth',2);
   hold on;
end
title('Evolution of U in the pipe', 'FontSize',14);
xlabel('r/R', 'FontSize',16); 
ylabel('U/U_{mean}', 'FontSize',16);
grid on,
axis([-1 1 0 max(max(UU))]);
%------------------------------------------------------------
%% Compute Fidot in a function DFlux
%------------------------------------------------------------
function Fidot = DFlux(~,Fi,param)
% Parameters 
eps=param(1);
N=param(2);
ratioaR=param(3);
lambda2=param(4);
lambda3=param(5);
r= (0:1/N:1)';
% Evaluate etan
etan=eta_n(Fi);
% Evaluate shear rate
[~,gammadot] = PipeAlgebric(Fi,r,ratioaR);
% Compute Flux
flux=eps*((1-Fi).^4).*(lambda2*gradient(abs(gammadot).*etan,1/N).*r + (lambda2-lambda3)*etan.*abs(gammadot));
% Impose zero flux on pipe wall and pipe center
flux(1)=0;
flux(end)=0;
Fidot=gradient(flux,1/N)./(r+10^(-5));
%------------------------------------------------------------
function [etan]=eta_n(phi)
%------------------------------------------------------------
% Correlations from rheology of Morris and Boulay
%------------------------------------------------------------ 
%------------------------------------------------------------ 
% input_args: volume fraction \phi
% output_args: etan
%------------------------------------------------------------
% Maximun packing fraction
phim=0.585;
% Coefficient Kn
Kn=0.75;
% Morris-Boulay correlation for etan
etan= Kn*((phi/phim).^2)./((1-phi/phim).^2);
%------------------------------------------------------------
function [etas]=eta_s_Morris(phi)
%------------------------------------------------------------
% Correlations from rheology of Morris and Boulay
%------------------------------------------------------------ 
%------------------------------------------------------------ 
% input_args: volume fraction \phi
% output_args: etas
%------------------------------------------------------------
% Maximun packing fraction 
phim=0.585;  
% Coefficient Ks
Ks=0.1;
% Morris-Boulay correlation for etas
etas= 1+((5/2).*phi./(1-phi/phim))+(Ks.*((phi/phim).^2)./((1-phi/phim).^2));
%------------------------------------------------------------
function [U,gammadot] = PipeAlgebric(phi,r,ratioaR)
%------------------------------------------------------------
% Compute the shear rate in a suspension pipe flow
% 4 + (1/r) \frac{\partial r \eta_s*gammadot}{\partial r}=0
% velocity normalized by Umax=K*R^2/(4* eta_f)
% r-coordinate normalized by R (pipe radius)
%------------------------------------------------------------
%input_args: the volume fraction \phi
%output_args: U and gammadot=dU/dr 
%------------------------------------------------------------
% Morris correlation for etas
etas= eta_s_Morris(phi);
%------------------------------------------------------------
% Compute gammadot = -2*r/eta_s
gammadot=-2*r./etas;
%------------------------------------------------------------
% Velocity field: U=\int_{0}^r gammadot dr - \int_{0}^1 gammadot dy
U=zeros(1,length(r));
% Velocity at pipe center: U(0)=- \int_{0}^1 gammadot dy 
U(1)=trapz(r,2*r./etas);
for i=2:1:length(r)
    A=gammadot;
    A(i+1:length(r))=[];
    B=r;
    B(i+1:length(r))=[];
    U(i)=trapz(B,A)+U(1); 
end
%------------------------------------------------------------
%Compute Umean = int_0^1 U*2*r dr
Umean=trapz(r,2*U'.*r);
% Compute velocity U averaged by the mean to obtain constant flow rate
U=U/Umean;
% Compute shear stress = etas* gammadot (Umax=1/Umean)
gammadot=(gammadot+1e-6)./Umean; 
% Add non local stress contribution = (a/R)^n * U/R with n=1 or 2
%gammadot= abs(gammadot)+ ratioaR^1/Umean;