function SBMPipeMorris_DynamicwC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate the dynamic particle distribution for a steady flow through 
% a tube using the suspension balance model.  This versio of the code
% calculates the fluxes in terms of the ratio of normal and shear stresses;
% a constraint is applied to keep the volume fraction from surpassing
% maximum random packing (see function PhiDot for more information).
%--------------------------------------------------------------------------
% The code is made dimensionless using a 
% length-scale = R (the pipe radius) and a
% velocity-scale = Umean (= Q/S = flow-rate / pipe section).
%--------------------------------------------------------------------------
% Correlations for the rheology are those proposed in 
% Morris, J., & Boulay, F. (1999) Journal of Rheology, 43(5), 1213-1237.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%__________________________________________________________________________
% BEGIN: USER DEFINED PARAMETERS
% Initial volume fraction (assumed uniform)
phiIN = 0.3;
% Input the maximum particle fraction:
phimax = 0.585;
% The ratio of particle radius to pipe radius
% 2mm particle: .201/1.64   1mm particle .105/1.64
ratioaR=.201/1.64; 
% Number of nodal points for the spatial discretization
nnode = 100;
% Final time: the mean flow velocity is held constant so that
% a time of one corresponds to a mean displacement through the
% tube of one radius.
Tfinal = 50.0;
% Interval for outputting results
Tint = 5.0;
%__________________________________________________________________________
% END: USER DEFINED PARAMETERS

% Calculate parameters from the user inputs.
[phi0,grids,param] = CalcParams(phiIN,phimax,ratioaR,nnode);
% Integrate using the ODE45 solver which uses the
% explicit Runge-Kutta (4,5) formula with variable time-step.
tspan = 0:Tint:Tfinal;
options = odeset('RelTol',1e-4);
[~,phi]=ode45(@DFlux,tspan,phi0,options,grids,param);
% Plot the volume fraction:
PlotPhi(grids.rnode,tspan,phi);
% Plot the velocity profile
%PlotVel(phi,tspan,grids,param);



function PhiDot = DFlux(~,phi,grids,param)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluate d(phi)/dt for each of the finite volumes, of which there are 
% nnodes.  The discretization is evenly spaced over the radial dimension,
% which ranges from 0 to 1 and the flux in and out of each element is 
% calculated at the mid-point between the nodes.  To avoid singularities,
% the flux at each of these points,
%
% \frac{2 a^2 f(\phi)}{9} \left[ 
%    \frac{\partial \eta_{rr} \dot{\gamma} }{\partial r} +
%    \frac{ \dot{\gamma} }{r} \left( \eta_{rr}-\eta_{\theta\theta} \right)
%                                                                  \right]
%
% has been rewritten in terms of the ratio of the normal and shear 
% components of the viscosities,
%
% \frac{G a^2 f(\phi)}{9} \left[ \lambda{2} r *
%    \frac{ \partial (\eta_o / \eta_s) }{ \partial r } *
%    \frac{ \partial \phi }{ \partial r } + 
%    \frac{\eta_o }{\eta_s} \left( 2\lambda_2 - \lambda_3 \right)  \right]
%
% where the relation \dot{\gamma} = \frac{G r}{2 \eta_s} has been used.
% The fluxes in and out of each midpoint are multiplied by the area
% and then divided by the volume of each element to give the change in
% volume fraction with time for each node i:
%
% \frac{d \phi_i}{dt} = \left[ (F*A)_{i-1/2} - (F*A)_{i+1/2} \right] / V-i
%
% where F indicates the flux evaluation, A the area, and V the volume.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Unpack the structures 'grids' and 'param':
    rnode = grids.rnode;
    rnA = grids.rnA;
    rflux = grids.rflux;
    rfA = grids.rfA;
    nnode = param.nnode;
    cflux = param.cflux;
    phimax = param.phimax;

% Evaluate the volume fraction, its derivative, and the rheology
% at the boundaries between the elements:
% estimate the value of phi between each node by interpolation
    phimid = (phi(1:nnode-1) + phi(2:nnode))/2;
% use a finite difference to estimate the gradient of phi 
    gradphi = (phi(2:nnode) - phi(1:nnode-1))*nnode;    
% calculate the rheology for each value of phi at the midpoints
    [ratio,dratio,vRZ,vl2,vl2m3] = evalratio(phimid,phimax);

% Calculate the pressure gradient to scale the shear rate so 
% that the displacement in the tube remains constant:
    [~,valG] = evalvelocity(phi,phimax,rnode,rnA,nnode);
    
% Calculate the flux*area in the radial direction at the edge of 
% each element (note that there is not flux through the center 
% at r=0 or through the wall of the tube at r=1):
    FtA = vl2m3*ratio'+vl2*rflux(2:nnode).*(gradphi.*dratio)';
    FtA = 2.0*cflux*valG*vRZ.*(rflux(2:nnode).*FtA)';
    
% Apply Constraint: Do not allow the volume fraction to exceed maximum 
% packing by setting any positive flux into a volume to zero if the
% volume fraction is larger than maximum packing (with a small offset):
    sgn = (1-sign(phi(1:nnode-1)-(phimax-0.0001)));
    Fout = FtA;
    Fout(Fout<0) = [0];
    FtA = Fout+.5*(FtA-Fout).*sgn;
    Fout = FtA;
    Fout(Fout>0) = [0];
    FtA = Fout+.5*(FtA-Fout).*sgn;
    
% Impose the boundary conditions of no flux at r=0 and r=1:
    FtA = [0;FtA;0];
% Combine the fluxes in and out of each element and divide by the 
% volume (area) of each element: this is the estimated value of 
% d\phi / dt for each node.
    PhiDot = (FtA(1:nnode)-FtA(2:nnode+1))./rfA';
        

    
function [vel,valG] = evalvelocity(phi,phimax,rnode,rnA,nnode)    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluate the velocity and pressure gradient, which is needed to scale 
% the shear rate evaluations for calculating the time-evolution of the
% concentration profile.  Note that the volumetric flowrate in the tube,
% not the pressure gradient, is held constant.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Solve for pressure driven flow in a circular tube with a viscosity
% that depends upon the radial position due to the inhomogeneity in 
% the particle conentration (\phi):
%
% \frac{\partial}{\partial r} \left( 
%      \eta_s r \frac{\partial u}{\partial r} \right) = Gr
%
% To solve, the velocity is discretized in the radial direction on a grid
% that is staggered relative to that used for the volume fraction, the
% momentum is conserved over each element, and a finite difference is used
% to estimate the gradient of the velocity at each point:
%
%   (\eta_s r)_{i-1/2} u_{i-1} - 
% ( (\eta_s r)_{i-1/2} + (\eta_s r)_{i+1/2} ) u_i +
%                        (\eta_s r)_{i+1/2} u_{i+1}
%                            = G/2 *( (r_{i+1/2})^2 - (r_{i-1/2})^2 ) * dr 
%
% where i indicates the node number and shifting by 1/2 indicates an
% evaluation at a midpoint between the nodes.  The spacing dr=1/nnode.

% Compute the right hand side (vector of length nnode+1):
    rhs = rnA/(2.0*nnode);
% Eevaluate the shear viscosity at each point and multiply by positions:    
    mtemp = evalviscs(phi,phimax).*rnode';
% Now form the tridiagonal matrix gm that multiplies the velocities:    
    gm = diag(mtemp,-1)+diag(mtemp,1);
    gm = gm - diag(sum(gm'));
% enforce the boundary condition of vel=0 at r=1:    
    rhs(nnode+1) = 0;
    gm(nnode+1,nnode+1) = 1;
    gm(nnode+1,nnode) = 0;
% solve the tridiagonal system of equations to give the velocities
    vel = gm\rhs';   

% The velocity profile for tube flow is G*R^2*(1- r^2)/(4*mu). Computing 
% the volumetric flow and dividing by the total area (pi*R^2) gives the 
% mean dispacement per unit time.  Holding the mean flow rate constant 
% at a value of 1 is equivalent, dimensionally, to the fluid traveling 
% one R in the dimensionless time of 1, regardless of the viscosity 
% distribution.  This is the correct control for the experiment, as we 
% control the displacement using a piston, not the pressure!
% First integrate the velocity to give the flow rate and divide by area
% to calculate the pressure gradient scaling:
    valG=1.d0/(rnA*vel);
% Scale velocities so that the mean displacement is 1 at all times:
    vel = valG*vel;     

    



function [ratio,dratio,vRZ,vl2,vl2m3] = evalratio(phi,phimax)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluate the ratio and derivative with respect to \phi of the 
% normal and shear stresses (viscosities): (\eta_n) / (\eta_s)
% Here, the rheology as defined in the work of Morris & Boulay 
% (J.Rheol., 1999) is being used in the calculation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Define constants for the rheology: Note that phimax is already defined
% as a user input, vl2 is \lambda_2, the coefficient for \Sigma_{rr}, and
% vl3 is \lambda_3, the coefficient of \Sigma_{\theta\theta}.
vl2 = 0.8;
vl3 = 0.5;
vl2m3 = 2.0*vl2-vl3;
vkn = 0.75;
vks = 0.1;

% Calculate (\eta_s/) \ (eta_n) to alleviate singularity issues 
% and then invert to give (\eta_n) / (\eta_s).
pr = phi/phimax;
ratio = (1.0-pr)./pr;
ratio = ratio.*ratio+2.5*ratio.*(phi./pr)+vks;
ratio = vkn./ratio;

% Calculate the derivative of the ratio (\eta_n) / (\eta_s) with
% respect to the volume fraction \phi.  To do so, use the identity
% d (eta_n/eta_s)/dphi = -(eta_s/eta_n)^2*d(eta_s/eta/n)/dphi
prmo = 1.0-pr;
pr2 = pr.*pr;
dratio = (2.0/phimax*(1+prmo./pr)+2.5).*(prmo./pr2)+2.5./pr; 
dratio = dratio.*(ratio.*ratio)/vkn;

% Evaluate the Richardson-Zaki law with an exponent of n=4:
vRZ = (1.0-phi).^4; 



function [etas] = evalviscs(phi,phimax)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluate the shear viscosity explicitly for calculation of the velocity
% profile and the scaling of the pressure gradient.  Here, using the 
% rheology as defined in the work of Morris & Boulay (J.Rheol., 1999).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

etas = 1.0-phi./phimax;
etas = 1.0+2.5*phi./etas+0.1*(phi./(phimax.*etas)).^2;



function [phi0,grids,param] = CalcParams(phiIN,phimax,ratioaR,nnode)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculate the constant parameters needed in the integration of the
% concentration profile.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Nodal positions for the evaluation of phi: there are a total
% of nnode elements over each of which the concentration 
% phi is held constant.  The vector rnode defines the center of
% each of the elements, which are evenly spaced by dr=1/nnode.
rnode = (0.5/nnode:1/nnode:1-.5/nnode);
% Nodal positions where both the fluxes for the particle conservation
% equation and the velocities are calculated.  These nodes are staggered
% with the grid that defines the positions for calculation of phi and 
% there are nnode+1 such positions.
rflux = (0:1/nnode:1);
% Calculate the area (normalized by pi) of each element over which 
% phi is discretized.
rfA = rflux(2:nnode+1).*rflux(2:nnode+1)-rflux(1:nnode).*rflux(1:nnode);
% Calculate the area (normalized by pi) between each of the flux 
% evaluation points.
rnA(1) = rnode(1)*rnode(1);
rnA(2:nnode) = rnode(2:nnode).*rnode(2:nnode)-rnode(1:nnode-1).*rnode(1:nnode-1);
rnA(nnode+1) = 1.0 - rnode(nnode)*rnode(nnode);
% The paramater that depends on the ratio of particle to tube radius.
cflux = ratioaR^2/9.0;
% Initialize the volme fraction for each finite volume: currently 
% set to a constant value, but that could be altered here.
phi0 = phiIN*ones(nnode,1);

% Place the commonly used variables into the structures 'grids' and
% 'param' for ease of passing between functions.
grids = struct('rnode',rnode,'rnA',rnA,'rflux',rflux,'rfA',rfA);
param = struct('nnode',nnode,'cflux',cflux,'phimax',phimax); 



function PlotPhi(rnode,tspan,phi)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the volume fraction \phi across the radius of the tube (-1<r<1) 
% as a function of time (tspan).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    figure;
    for k = 1:1:length(tspan)
% Plot from 0 to 1
        plot(rnode,phi(k,:),'-r','LineWidth',2);
        hold on;
% Plot symmetric part from 0 to -1
        plot(-rnode,phi(k,:),'-r','LineWidth',2);
        hold on;
    end
%title('Evolution of \phi in the pipe', 'FontSize',14);
    xlabel('r/R', 'FontSize',16); 
    ylabel('\phi', 'FontSize',16);
    grid on,
    axis([-1 1 0.2 0.6]);



function PlotVel(phi,tspan,grids,param)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the volume fraction \phi across the radius of the tube (-1<r<1) 
% as a function of time (tspan).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% unpack needed variables from grids and param
    rnode = grids.rnode;
    rnA = grids.rnA;
    r = grids.rflux;
    nnode = param.nnode;    
    phimax = param.phimax;

    figure;
    UU = zeros(length(tspan),length(r));
    for k=1:1:length(tspan)
% must recalculate the velocity for each volume fraction
        ptemp = phi(k,:)';
        [U,~] = evalvelocity(ptemp,phimax,rnode,rnA,nnode);
% Plot from 0 to 1
        plot(r,U, '-b','LineWidth',2);
        UU(k,:)=U;
        hold on;
% Plot symmetric part from 0 to -1
        plot(-r,U, '-b','LineWidth',2);
        hold on;
    end
    title('Evolution of U in the pipe', 'FontSize',14);
    xlabel('r/R', 'FontSize',16); 
    ylabel('U/U_{mean}', 'FontSize',16);
    grid on,
    axis([-1 1 0 max(max(UU))]);



% END OF PROGRAM