function SBMPipeMorris_Steady
%---------------------------------------------------------------
% Steady state concentration profile in pipe flow given by the Suspension Balance Model
% using frictional rheology of
% Morris, J., & Boulay, F. (1999) Journal of Rheology, 43(5), 1213?1237.
%---------------------------------------------------------------
% Central jammed region of the pipe delimited by 
% r_jam =[\mu(\phi_m)/\mu_w]^{1/(2-\lambda_3/\lambda_2)}
% Outside this jammed region \phi determined by inverting
% \mu (\phi) = \eta_s (\phi)/\eta_n(\phi) = \mu_w r^(2-\lambda_3/\lambda_2)
% Insuring total volume fraction = initial volume fraction provides \mu_w
%---------------------------------------------------------------
% length-scale = R = pipe radius
% velocity-scale = Umean = Q/S = flow-rate / pipe section
%------------------------------------------------------------
%% Clear
%------------------------------------------------------------
close all;
%------------------------------------------------------------
%% Initial data
%------------------------------------------------------------
% Lambda2
lambda2= 0.8;
% Lambda3
lambda3= 0.5;
% Maximun packing fraction 
phim=0.585;   
% Discretization
N=100;
% Initial volume fraction
phi0=0.3;
Fi0=phi0*ones(N+1,1);
% Space-range
r=(0:1/N:1)';
%------------------------------------------------------------
%% Determination of muw
%------------------------------------------------------------
% Find muw for which total volume fraction = initial volume fraction phi0
%---------------------------------------------------------------
% Parameters
%---------------------------------------------------------------
% Optimization parameter = muw 
muw0=1;
% Other parameters to pass into the function
param=[lambda2;lambda3;phim;N];
%--------------------------------------------------------------- 
% Unconstrained nonlinear optimization
%---------------------------------------------------------------
% finds the minimum of function LSFimean (least square)
% starting at initial estimate muw0
% and return the local minimizer of the function
%---------------------------------------------------------------
muw= fminsearch(@(muw) LSFimean(phi0,param,muw),muw0);
%------------------------------------------------------------
%% Jamming location
%------------------------------------------------------------ 
rjam=(mu_Morris(phim)/muw)^(1/(2-lambda3/lambda2));
%------------------------------------------------------------
%% Concentration profile
%------------------------------------------------------------
y=muw*r.^(2-lambda3/lambda2);
Fi= zeros(N+1,1);
for k = 1:1:length(y) 
    if (r(k)>rjam)
    % Given y and guess x0 for the inverse
    % x = fzero( @(x)(f(x)-y), x0 ); 
    FiFi = fzero(@(FiFi)(mu_Morris(FiFi)-y(k)),phim);  
    else
    FiFi=phim;
    end
Fi(k)=FiFi;
end
%------------------------------------------------------------
%% Mean concentration
%------------------------------------------------------------
% Compute Fimean = int_0^1 Fi*2*r dr
phimean=trapz(r,2*Fi.*r);
Fimean=phimean*ones(N+1,1);
%------------------------------------------------------------
%% Plot volume fraction
%------------------------------------------------------------
figure;
hold on;
% Plot from 0 to 1
plot(r,Fi0, '-k',r,Fimean, '-k', r,Fi, '-k','LineWidth',2);
hold on;
% Plot symmetric part from 0 to -1
plot(-r,Fi0, '-k',-r,Fimean, '-k',-r,Fi, '-k','LineWidth',2);
grid on, axis([-1 1 0 0.6])
title(['Evolution of \phi  in the pipe: r_{jam} = ',num2str(rjam),' and \mu_w =',num2str(muw)], 'FontSize',14);
xlabel('r/R', 'FontSize',16);
ylabel('\phi', 'FontSize',16);
%------------------------------------------------------------
%% Plot velocity
%------------------------------------------------------------
figure;
[U] = PipeAlgebric(Fi,r);
% Plot from 0 to 1
plot(r,U, '-k','LineWidth',2);
hold on;
% Plot symmetric part from 0 to -1
plot(-r,U, '-k','LineWidth',2);
grid on, axis([-1 1 0 2]);
title('Evolution of U in the pipe', 'FontSize',14);
xlabel('r/R', 'FontSize',16); 
ylabel('U/U_{mean}', 'FontSize',16);
%------------------------------------------------------------
function [mu]=mu_Morris(phi)
%------------------------------------------------------------ 
% Compute correlation for mu=etas/etan
% using rheology of Morris and Boulay
% mu= ((1-phi/phim).^2 +(5/2)*phi.*(1-phi/phim)+ Ks.*((phi/phim).^2))./Kn*((phi/phim).^2)
%------------------------------------------------------------ 
% input_args: volume fraction \phi
% output_args: mu=etas/etan
%------------------------------------------------------------
% Maximun packing fraction
phim=0.585;
% Morris-Boulay correlation
% Coefficient Kn and Ks
Kn=0.75;
Ks=0.1;
% Compute mu
mu= ((1-phi/phim).^2)./(Kn*(phi/phim).^2) +(5/2)*phim.*(1-phi/phim)./(Kn*(phi/phim))+ Ks/Kn;
%------------------------------------------------------------
function [etas]=eta_s_Morris(phi)
%------------------------------------------------------------
% Correlations from rheology of Morris and Boulay
%------------------------------------------------------------ 
%------------------------------------------------------------ 
% input_args: volume fraction \phi
% output_args: etas
%------------------------------------------------------------
% Maximun packing fraction 
phim=0.585;  
% Coefficient Ks
Ks=0.1;
% Morris-Boulay correlation for etas
etas= 1+((5/2).*phi./(1-phi/phim))+(Ks.*((phi/phim).^2)./((1-phi/phim).^2));
%------------------------------------------------------------
function E =LSFimean(phi0,param,muw)
%------------------------------------------------------------ 
% Parameters passed into the function
%------------------------------------------------------------ 
lambda2=param(1);%2.*(phi-0.1);
lambda3=param(2);%1.2.*(fi-0.1);
phim=param(3);
N=param(4);
% Space-range
r=(0:1/N:1)';
%------------------------------------------------------------ 
% Jamming location
%------------------------------------------------------------ 
rjam=(mu_Morris(phim)/muw)^(1/(2-lambda3/lambda2));
%------------------------------------------------------------
% Concentration profile
%------------------------------------------------------------
y=muw*r.^(2-lambda3/lambda2);
Fi= zeros(N+1,1);
for k = 1:1:length(y) 
    if (r(k)>rjam)
    %Given y and guess x0 for the inverse
    %x = fzero( @(x)(f(x)-y), x0 ); 
    FiFi = fzero(@(FiFi)(mu_Morris(FiFi)-y(k)),phim);  
    else
    FiFi=phim;
    end
Fi(k)=FiFi;
end
%------------------------------------------------------------
% Mean concentration
%------------------------------------------------------------
%Compute Fimean = int_0^1 Fi*2*r dr
Fimean=trapz(r,2*Fi.*r);
%------------------------------------------------------------
%Least Mean Square
%------------------------------------------------------------
E=(Fimean-phi0)*(Fimean-phi0)';
%------------------------------------------------------------
function [U] = PipeAlgebric(phi,r)
%------------------------------------------------------------
% Compute the shear rate in a suspension pipe flow
% 4 + (1/r) \frac{\partial r \eta_s*gammadot}{\partial r}=0
% velocity normalized by Umax=K*R^2/(4* eta_f)
% r-coordinate normalized by R (pipe radius)
%------------------------------------------------------------
%input_args: the volume fraction \phi
%output_args: U and gammadot=dU/dr 
%------------------------------------------------------------
% Compute etas
etas= eta_s_Morris(phi);
%------------------------------------------------------------
% Compute gammadot = -2*r/eta_s
gammadot=-2*r./etas;
%------------------------------------------------------------
% Velocity field: U=\int_{0}^r gammadot dr - \int_{0}^1 gammadot dy
U=zeros(1,length(r));
% Velocity at pipe center: U(0)=- \int_{0}^1 gammadot dy 
U(1)=trapz(r,2*r./etas);
for i=2:1:length(r)
    A=gammadot;
    A(i+1:length(r))=[];
    B=r;
    B(i+1:length(r))=[];
    U(i)=trapz(B,A)+U(1); 
end
%------------------------------------------------------------
%Compute Umean = int_0^1 U*2*r dr
Umean=trapz(r,2*U'.*r);
% Compute velocity U averaged by the mean to obtain constant flow rate
U=U/Umean;